/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.certsecurity.custom.ejb;

import java.util.HashMap;
import java.util.Map;
import javax.naming.ldap.Control;

/**
 *
 * <p>Class that represent a bind result. It is just the status and the
 * controls returned by the server and meaningful for the status</p>
 * 
 * @author ricky
 */
public class UserBindResult extends UserBindStatus {
    
    /**
     * Control map returned by the server and meaningful for this
     * status.
     */
    protected Map<String,Control> controlValues;
    
    /**
     * Constructs a new UserBindResult with the status and the controls.
     * 
     * @param status The status of this result
     * @param controls The controls returned by the server
     */
    protected UserBindResult(UserBindStatus status, Control[] controls) {
        this(status);
        if (controls != null) {
            for (Control c : controls) {
                if (this.controls.contains(c.getID())) {
                    controlValues.put(c.getID(), c);
                }
            }
        }
    }
    
    /**
     * Constructor only with the status.
     * @param status The status
     */
    public UserBindResult(UserBindStatus status) {
        super(status);
        this.controlValues = new HashMap<String, Control>();
    }

    /**
     * Return the map of controls that this status saved. The result only
     * saves the controls defined by the status. 
     * @return The map of the status
     */
    public Map<String, Control> getControlValues() {
        return controlValues;
    }
}
