/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.certsecurity.custom.jsfbean;

import java.io.IOException;
import java.util.Properties;
import javax.enterprise.inject.Produces;
import javax.enterprise.inject.spi.InjectionPoint;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The Factory to initialize the values of the beans. It just reads all
 * the initialization parameters from a "beans-init-values.properties" 
 * file situated somewhere in the classpath.
 * 
 * See: http://weblogs.java.net/blog/jjviana/archive/2010/05/18/applicaction-configuration-java-ee-6-using-cdi-simple-example
 * 
 * 
 * @author ricky
 */
public class ConfigurationFactory {

    /**
     * logger for the class
     */
    static private final Logger logger = LoggerFactory.getLogger(ConfigurationFactory.class);
    /**
     * name of the properties file: "beans-init-values.properties"
     */
    public final static String PROPS_FILE_NAME = "beans-init-values.properties";
    /**
     * static properties file read at initialization
     */
    private final static Properties configProperties = getProperties();

    /**
     * Read the properties from the classpath searching the "beans-init-values.properties"
     * @return 
     */
    private synchronized static Properties getProperties() {
        Properties props = new Properties();
        try {
            props.load(ConfigurationFactory.class.getClassLoader().getResourceAsStream(PROPS_FILE_NAME));
        } catch (IOException ex) {
            logger.debug("Error loading configuration for beans!", ex);
            throw new RuntimeException(ex);
        }
        return props;
    }

    /**
     * Read the value using the classname of the beam and the property name.
     * Example: 
     *   For a declared property in "mypackage.MyClass":
     *   @Inject @ConfigProperty private String fieldName;
     * It first searches the property: "myPackage.MyClass.fieldName" and if it
     * does not exists the "MyClass.fieldName" (full qualified and simple class 
     * name).
     * @param p The injection point where the @Inject was declared
     * @return The value read from the proiperties file or null
     */
    public @Produces @ConfigProperty String getConfiguration(InjectionPoint p) {
        String configKey = new StringBuilder(p.getMember().getDeclaringClass().getName()).append(".").append(p.getMember().getName()).toString();
        if (configProperties.getProperty(configKey) == null) {
            configKey = new StringBuilder(p.getMember().getDeclaringClass().getSimpleName()).append(".").append(p.getMember().getName()).toString();
            if (configProperties.getProperty(configKey) == null) {
                configKey = p.getMember().getName();
            }
        }
        logger.debug("Config key= " + configKey + " value = " + configProperties.getProperty(configKey));
        return configProperties.getProperty(configKey);
    }

    /**
     * Same but transforming into a boolean
     * 
     * @param p The injection point 
     * @return The boolean value
     */
    public @Produces @ConfigProperty boolean getConfigurationDouble(InjectionPoint p) {
        String val = getConfiguration(p);
        return Boolean.parseBoolean(val);
    }
}
